"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../sfpowerkit");
class BaseMetadataRetriever {
    constructor(org, tooling = false) {
        this.org = org;
        this.tooling = tooling;
        this.isLimitBasedQueryRetrieval = false;
        this.dataLoaded = false;
        this.cacheFileName = "";
    }
    setCountQuery(countQuery, fetchSize) {
        this.isLimitBasedQueryRetrieval = true;
        this.countQuery = countQuery;
        this.fetchSize = fetchSize;
    }
    setQuery(query) {
        this.query = query;
        if (this.isLimitBasedQueryRetrieval)
            this.queryWithOffsetsAndLimit = this.query.concat(` LIMIT ${this.fetchSize} OFFSET 0`);
    }
    async getObjects() {
        let records = [];
        const conn = this.org.getConnection();
        // Not Limit and Offset, based so old method
        if (!this.isLimitBasedQueryRetrieval) {
            sfpowerkit_1.SFPowerkit.log(`Method: isTooling :  ${this.tooling}, QUERY:  ${this.query}`, core_1.LoggerLevel.TRACE);
            let result;
            // Query the org
            if (this.tooling) {
                result = await conn.tooling.query(this.query);
            }
            else {
                result = await conn.query(this.query);
            }
            records.push(...result.records);
            while (!result.done) {
                result = await this.queryMore(result.nextRecordsUrl);
                records.push(...result.records);
            }
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`Method: isToolingandLimitBasedQueryRetrieval : true, QUERY:  ${this.query}`, core_1.LoggerLevel.TRACE);
            let retrievedRecordSize = 0;
            let offset = 0;
            this.totalSize = await this.getCount();
            while (retrievedRecordSize < this.totalSize) {
                sfpowerkit_1.SFPowerkit.log(`To Retrieve Total Size:  ${this.totalSize},Retrieved Size:   ${retrievedRecordSize} , Current Offset: ${offset}`, core_1.LoggerLevel.TRACE);
                let result;
                sfpowerkit_1.SFPowerkit.log(this.queryWithOffsetsAndLimit, core_1.LoggerLevel.TRACE);
                result = await conn.tooling.query(this.queryWithOffsetsAndLimit);
                retrievedRecordSize += result.totalSize;
                records.push(...result.records);
                offset = offset + this.fetchSize;
                this.queryWithOffsetsAndLimit = this.query.concat(` LIMIT ${this.fetchSize} OFFSET ${offset}`);
            }
        }
        return records;
    }
    async queryMore(url) {
        const conn = this.org.getConnection();
        let result;
        if (this.tooling) {
            result = await conn.tooling.queryMore(url);
        }
        else {
            result = await conn.queryMore(url);
        }
        return result;
    }
    async getCount() {
        sfpowerkit_1.SFPowerkit.log(`Count Query: ${this.countQuery}`, core_1.LoggerLevel.TRACE);
        let result = await this.org.getConnection().tooling.query(this.countQuery);
        sfpowerkit_1.SFPowerkit.log(`Retrieved count ${result.totalSize}`, core_1.LoggerLevel.TRACE);
        return result.totalSize;
    }
}
exports.default = BaseMetadataRetriever;
//# sourceMappingURL=baseMetadataRetriever.js.map