"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const xml2js = __importStar(require("xml2js"));
const _ = require("lodash");
const util = require("util");
const diffUtil_1 = __importDefault(require("./diffUtil"));
const parser = new xml2js.Parser({
    explicitArray: false,
    valueProcessors: [
        function (name) {
            if (name === "true")
                name = true;
            if (name === "false")
                name = false;
            return name;
        }
    ]
});
class SharingRuleDiff {
    static async generateSharingRulesXml(sharingRuleXml1, sharingRuleXml2, outputFilePath, objectName, destructivePackageObj, resultOutput) {
        const parseString = util.promisify(parser.parseString);
        let sharingRulesObj1 = {};
        let sharingRulesObj2 = {};
        if (sharingRuleXml1 !== "") {
            let parseResult = await parseString(sharingRuleXml1);
            sharingRulesObj1 = parseResult.SharingRules || {};
        }
        if (sharingRuleXml2 !== "") {
            let parseResult = await parseString(sharingRuleXml2);
            sharingRulesObj2 = parseResult.SharingRules || {};
        }
        let addedEditedOrDeleted = SharingRuleDiff.buildSharingRulesObj(sharingRulesObj1, sharingRulesObj2);
        SharingRuleDiff.writeSharingRule(addedEditedOrDeleted.addedEdited, outputFilePath);
        destructivePackageObj = SharingRuleDiff.buildDestructiveChangesObj(addedEditedOrDeleted.deleted, destructivePackageObj, objectName);
        SharingRuleDiff.updateOutput(addedEditedOrDeleted.addedEdited, resultOutput, objectName, "Deploy", outputFilePath);
        SharingRuleDiff.updateOutput(addedEditedOrDeleted.deleted, resultOutput, objectName, "Delete", "destructiveChanges.xml");
        return destructivePackageObj;
    }
    static updateOutput(sharingRulesObj, resultOutput, objectName, action, filePath) {
        sharingRulesObj.sharingCriteriaRules.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "SharingCriteriaRule",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
        sharingRulesObj.sharingOwnerRules.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "SharingOwnerRule",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
        sharingRulesObj.sharingTerritoryRules.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "SharingTerritoryRule",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
    }
    static ensureArray(sharingObj) {
        let keys = Object.keys(sharingObj);
        keys.forEach(key => {
            if (typeof sharingObj[key] === "object" &&
                !Array.isArray(sharingObj[key]) &&
                key !== "$") {
                sharingObj[key] = [sharingObj[key]];
            }
        });
        return sharingObj;
    }
    static buildSharingRulesObj(sharingRuleObj1, sharingRulesObj2) {
        let newSharingRuleObj = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            sharingCriteriaRules: [],
            sharingOwnerRules: [],
            sharingTerritoryRules: []
        };
        sharingRuleObj1 = SharingRuleDiff.ensureArray(sharingRuleObj1);
        sharingRulesObj2 = SharingRuleDiff.ensureArray(sharingRulesObj2);
        let deletedSharingObj = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            sharingCriteriaRules: [],
            sharingOwnerRules: [],
            sharingTerritoryRules: []
        };
        let addedDeleted = diffUtil_1.default.getChangedOrAdded(sharingRuleObj1.sharingCriteriaRules, sharingRulesObj2.sharingCriteriaRules, "fullName");
        newSharingRuleObj.sharingCriteriaRules = addedDeleted.addedEdited;
        deletedSharingObj.sharingCriteriaRules = addedDeleted.deleted;
        addedDeleted = diffUtil_1.default.getChangedOrAdded(sharingRuleObj1.sharingOwnerRules, sharingRulesObj2.sharingOwnerRules, "fullName");
        newSharingRuleObj.sharingOwnerRules = addedDeleted.addedEdited;
        deletedSharingObj.sharingOwnerRules = addedDeleted.deleted;
        addedDeleted = diffUtil_1.default.getChangedOrAdded(sharingRuleObj1.sharingTerritoryRules, sharingRulesObj2.sharingTerritoryRules, "fullName");
        newSharingRuleObj.sharingTerritoryRules = addedDeleted.addedEdited;
        deletedSharingObj.sharingTerritoryRules = addedDeleted.deleted;
        return {
            addedEdited: newSharingRuleObj,
            deleted: deletedSharingObj
        };
    }
    static buildDestructiveChangesObj(deletedSharing, destructivePackageObj, objectName) {
        let sharingCriteriaRules = _.find(destructivePackageObj, function (metaType) {
            return metaType.name === "SharingCriteriaRule";
        });
        if (sharingCriteriaRules === undefined &&
            deletedSharing.sharingCriteriaRules !== undefined &&
            deletedSharing.sharingCriteriaRules.length > 0) {
            sharingCriteriaRules = {
                name: "SharingCriteriaRule",
                members: []
            };
            destructivePackageObj.push(sharingCriteriaRules);
        }
        if (deletedSharing.sharingCriteriaRules !== undefined) {
            deletedSharing.sharingCriteriaRules.forEach(elem => {
                sharingCriteriaRules.members.push(objectName + "." + elem.fullName);
            });
        }
        let sharingOwnerRules = _.find(destructivePackageObj, function (metaType) {
            return metaType.name === "SharingOwnerRule";
        });
        if (sharingOwnerRules === undefined &&
            deletedSharing.sharingOwnerRules !== undefined &&
            deletedSharing.sharingOwnerRules.length > 0) {
            sharingOwnerRules = {
                name: "SharingOwnerRule",
                members: []
            };
            destructivePackageObj.push(sharingOwnerRules);
        }
        if (deletedSharing.sharingOwnerRules !== undefined) {
            deletedSharing.sharingOwnerRules.forEach(elem => {
                sharingOwnerRules.members.push(objectName + "." + elem.fullName);
            });
        }
        let sharingTerritoryRules = _.find(destructivePackageObj, function (metaType) {
            return metaType.name === "SharingTerritoryRule";
        });
        if (sharingTerritoryRules === undefined &&
            deletedSharing.sharingTerritoryRules !== undefined &&
            deletedSharing.sharingTerritoryRules.length > 0) {
            sharingTerritoryRules = {
                name: "SharingTerritoryRule",
                members: []
            };
            destructivePackageObj.push(sharingTerritoryRules);
        }
        if (deletedSharing.sharingTerritoryRules !== undefined) {
            deletedSharing.sharingTerritoryRules.forEach(elem => {
                sharingTerritoryRules.members.push(objectName + "." + elem.fullName);
            });
        }
        return destructivePackageObj;
    }
    static writeSharingRule(newSharingRulesObj, outputFilePath) {
        const builder = new xml2js.Builder({
            xmldec: { version: "1.0", encoding: "UTF-8", standalone: null }
        });
        let sharingRulesObj = {
            SharingRules: newSharingRulesObj
        };
        let xml = builder.buildObject(sharingRulesObj);
        fs.writeFileSync(outputFilePath, xml);
    }
}
exports.default = SharingRuleDiff;
//# sourceMappingURL=sharingRuleDiff.js.map