"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const metadataInfo_1 = require("../../metadata/metadataInfo");
const sfpowerkit_1 = require("../../../sfpowerkit");
const path = __importStar(require("path"));
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
const retrieveMetadata_1 = require("../../../utils/retrieveMetadata");
const profileRetriever_1 = __importDefault(require("../../metadata/retriever/profileRetriever"));
class ProfileActions {
    constructor(org, debugFlag) {
        this.org = org;
        if (this.org !== undefined) {
            this.conn = this.org.getConnection();
        }
        this.debugFlag = debugFlag;
        this.profileRetriever = new profileRetriever_1.default(org, debugFlag);
    }
    async getProfileFullNamesWithLocalStatus(profileNames) {
        let profilesStatus = {
            added: [],
            deleted: [],
            updated: []
        };
        let metadataFiles = metadataInfo_1.METADATA_INFO.Profile.files || [];
        //generate path for new profiles
        let profilePath = path.join(process.cwd(), await sfpowerkit_1.SFPowerkit.getDefaultFolder(), "main", "default", "profiles");
        if (metadataFiles && metadataFiles.length > 0) {
            profilePath = path.dirname(metadataFiles[0]);
        }
        else {
            //create folder structure
            fileutils_1.default.mkDirByPathSync(profilePath);
        }
        if (profileNames && profileNames.length > 0) {
            metadataFiles = [];
            for (let i = 0; i < profileNames.length; i++) {
                let profileName = profileNames[i];
                let found = false;
                for (let j = 0; j < metadataFiles.length; j++) {
                    let profileComponent = metadataFiles[j];
                    let oneName = path.basename(profileComponent, metadataInfo_1.METADATA_INFO.Profile.sourceExtension);
                    if (profileName === oneName) {
                        //metadataFiles.push(profileComponent);
                        profilesStatus.updated.push(profileComponent);
                        found = true;
                        break;
                    }
                }
                //Query the profile from the server
                let profiles = await retrieveMetadata_1.retrieveMetadata([{ type: "Profile", folder: null }], this.conn);
                if (!found) {
                    for (let k = 0; k < profiles.length; k++) {
                        if (profiles[k] === profileName) {
                            let newProfilePath = path.join(profilePath, profiles[k] + metadataInfo_1.METADATA_INFO.Profile.sourceExtension);
                            //metadataFiles.push(newProfilePath);
                            profilesStatus.added.push(newProfilePath);
                            found = true;
                            break;
                        }
                    }
                }
                if (!found) {
                    profilesStatus.deleted.push(profileName);
                }
            }
        }
        else {
            if (this.debugFlag)
                sfpowerkit_1.SFPowerkit.ux.log("Load new profiles from server into the project directory");
            // Query the org
            const profiles = await retrieveMetadata_1.retrieveMetadata([{ type: "Profile", folder: null }], this.conn);
            profilesStatus.deleted = metadataFiles.filter(file => {
                let oneName = path.basename(file, metadataInfo_1.METADATA_INFO.Profile.sourceExtension);
                return !profiles.includes(oneName);
            });
            profilesStatus.updated = metadataFiles.filter(file => {
                let oneName = path.basename(file, metadataInfo_1.METADATA_INFO.Profile.sourceExtension);
                return profiles.includes(oneName);
            });
            if (profiles && profiles.length > 0) {
                let newProfiles = profiles.filter(profileObj => {
                    let found = false;
                    for (let i = 0; i < profilesStatus.updated.length; i++) {
                        let profileComponent = profilesStatus.updated[i];
                        let oneName = path.basename(profileComponent, metadataInfo_1.METADATA_INFO.Profile.sourceExtension);
                        //escape some caracters
                        let onlineName = profileObj.replace("'", "%27");
                        onlineName = onlineName.replace("/", "%2F");
                        if (onlineName === oneName) {
                            found = true;
                            break;
                        }
                    }
                    return !found;
                });
                if (newProfiles && newProfiles.length > 0) {
                    if (this.debugFlag)
                        sfpowerkit_1.SFPowerkit.ux.log("New profiles founds");
                    for (let i = 0; i < newProfiles.length; i++) {
                        if (this.debugFlag)
                            sfpowerkit_1.SFPowerkit.ux.log(newProfiles[i]);
                        let newPRofilePath = path.join(profilePath, newProfiles[i] + metadataInfo_1.METADATA_INFO.Profile.sourceExtension);
                        //metadataFiles.push(newPRofilePath);
                        profilesStatus.added.push(newPRofilePath);
                    }
                }
                else {
                    sfpowerkit_1.SFPowerkit.ux.log("No new profile found, Updating existing profiles");
                }
            }
        }
        //metadataFiles = metadataFiles.sort();
        return Promise.resolve(profilesStatus);
    }
}
exports.default = ProfileActions;
//# sourceMappingURL=profileActions.js.map