"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const Logger = require("pino");
class SFPowerkit {
    static setLogLevel(logLevel) {
        logLevel = logLevel.toLowerCase();
        this.logger = Logger({
            name: "sfpowerkit",
            level: logLevel,
            prettyPrint: {
                levelFirst: true,
                colorize: true,
                translateTime: true,
                ignore: "pid,hostname" // --ignore
            }
        });
    }
    static async getProjectDirectories() {
        if (!SFPowerkit.projectDirectories) {
            SFPowerkit.projectDirectories = [];
            const dxProject = await core_1.SfdxProject.resolve();
            const project = await dxProject.retrieveSfdxProjectJson();
            let packages = project.get("packageDirectories") || [];
            packages.forEach(element => {
                SFPowerkit.projectDirectories.push(element.path);
                if (element.default) {
                    SFPowerkit.defaultFolder = element.path;
                }
            });
        }
        return SFPowerkit.projectDirectories;
    }
    static async getDefaultFolder() {
        if (!SFPowerkit.defaultFolder) {
            await SFPowerkit.getProjectDirectories();
        }
        return SFPowerkit.defaultFolder;
    }
    static setDefaultFolder(defaultFolder) {
        SFPowerkit.defaultFolder = defaultFolder;
    }
    static async getConfig() {
        if (!SFPowerkit.pluginConfig) {
            const dxProject = await core_1.SfdxProject.resolve();
            const project = await dxProject.retrieveSfdxProjectJson();
            let plugins = project.get("plugins") || {};
            let sfpowerkitConfig = plugins["sfpowerkit"];
            SFPowerkit.pluginConfig = sfpowerkitConfig || {};
        }
        return SFPowerkit.pluginConfig;
    }
    static async getApiVersion() {
        const dxProject = await core_1.SfdxProject.resolve();
        const project = await dxProject.retrieveSfdxProjectJson();
        return project.get("sourceApiVersion");
    }
    /**
     * Print log only if the log level for this commamnd matches the log level for the message
     * @param message Message to print
     * @param messageLoglevel Log level for the message
     */
    static log(message, logLevel) {
        switch (logLevel) {
            case core_1.LoggerLevel.TRACE:
                this.logger.trace(message);
                break;
            case core_1.LoggerLevel.DEBUG:
                this.logger.debug(message);
                break;
            case core_1.LoggerLevel.INFO:
                this.logger.info(message);
                break;
            case core_1.LoggerLevel.WARN:
                this.logger.warn(message);
                break;
            case core_1.LoggerLevel.ERROR:
                this.logger.error(message);
                break;
            case core_1.LoggerLevel.FATAL:
                this.logger.fatal(message);
                break;
        }
    }
    /**
     * Print log only if the log level for this commamnd matches the log level for the message
     * @param message Message to print in JSON Format
     * @param messageLoglevel  Log level for the message
     */
    static logJson(message, logLevel) {
        switch (logLevel) {
            case core_1.LoggerLevel.TRACE:
            case core_1.LoggerLevel.DEBUG:
                this.ux.logJson(message);
        }
    }
}
exports.SFPowerkit = SFPowerkit;
//# sourceMappingURL=sfpowerkit.js.map